/*
 * Copyright 2004, U.S. Robotics Corporation
 * All Rights Reserved.
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND U.S. Robotics GRANTS NO WARRANTIES OF
 * ANY KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. U.S.
 * Robotics SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: wan_detection.c,v 1.6 2005/09/02 19:46:42 dtutuianu Exp $
 */
#ifdef WEBS
#include <webs.h>
#include <uemf.h>
#include <ej.h>
#else /* !WEBS */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <assert.h>
#include <httpd.h>
#endif /* WEBS */


#include <typedefs.h>
#include <proto/ethernet.h>
#include <bcmnvram.h>
#include <bcmutils.h>
#include <shutils.h>
#include <netconf.h>
#include <nvparse.h>
#include <wlutils.h>
#include <bcmcvar.h>
#include <ezc.h>
#include <bcmconfig.h>


#if defined(linux)

#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <sys/klog.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <net/if.h>

typedef u_int64_t u64;
typedef u_int32_t u32;
typedef u_int16_t u16;
typedef u_int8_t u8;
#include <linux/ethtool.h>
#include <linux/sockios.h>
#include <net/if_arp.h>

#ifdef __CONFIG_NAT__
#define wan_prefix(unit, prefix)	snprintf(prefix, sizeof(prefix), "wan%d_", unit)
static volatile int force_detection = 0;

/*
static void local_run_pppoe()
{
    char tmp[100];
    char *usr, *pwd;

    usr = nvram_get("wan_pppoe_username");
    pwd = nvram_get("wan_pppoe_passwd");

    if (usr && pwd)
    {
        cprintf("run pppoe...\n");

	sprintf(tmp, "pppoecd vlan1 -u %s -p %s -r 1492 -t 1492 -i 0 -U 0&",usr,pwd);
	system(tmp);
    }
} */

static int
local_check_pppoelogin(webs_t wp, char *usr, char *pwd)
{
    //char prefix[] = "wanXXXXXXXXXX_";
    char tmp[100];
    int ret = 0;
    FILE *fp;
    char *str;
    int pid,t;

    if (!usr || !pwd)
	return -1;

    unlink("/tmp/USRpr");			
    if ((str = file2str("/var/run/USRpppoecd.pid"))) 
    {
	pid = atoi(str);
	free(str);
	cprintf("Send SIGTERM (%d) to pppoecd's process %d...\n",SIGTERM, pid);
	kill(pid, SIGTERM);
	sleep(1); /* wait for server detection from udhcpd */
    }
    sprintf(tmp, "pppoecd vlan1 -D 2 -u %s -p %s -r 1492 -t 1492 -i 0 -U 0",usr,pwd);
    cprintf("launch %s\n",tmp);
    system(tmp);
    
    t = 0;
    
    while (t < 10)
    {
	if ((fp = fopen("/tmp/USRpr", "r")))
    	    break;
	t++;
	cprintf("wait here: %d\n", t);
        sleep(1); /* wait for pppoe server detection result */
    }
    if (fp)
    {
        if (fread(tmp, 1, 1, fp) == 1) 
	{
    	if (!strncmp(tmp, "2", 1))
	        ret = 2;
	}
        fclose(fp);
	unlink("/tmp/USRpr"); 
    }
    cprintf("pppoe result = %d\n",ret);
/*    if (nvram_match("wan_proto", "pppoe")) run_pppoe(); */
    return ret;
}


static int
local_check_pppoe(int eid, webs_t wp, int argc, char_t **argv)
{
	char *wan_ifname;
	char tmp[100], prefix[] = "wanXXXXXXXXXX_";
	int ret = 0;
	FILE *fp;
	struct ifreq ifr;
	//struct ethtool_cmd ecmd;
	int s;
        char *str;
	int pid, t;
	int unit;
	
	if ((unit = atoi(nvram_safe_get("wan_unit"))) < 0)
		unit = 0;
	wan_prefix(unit, prefix);

	cprintf("Check pppoe connection...\n");
	/* PPPoE connection status */
	if (nvram_match("wan_proto", "pppoe")) 
	{
	    cprintf("pppoe is selected as default...\n");
	    wan_ifname = nvram_safe_get(strcat_r(prefix, "pppoe_ifname", tmp));
	    if ((fp = fopen(strcat_r("/tmp/ppp/link.", wan_ifname, tmp), "r"))) 
	    {
		cprintf("ppp_link opened: %s\n", wan_ifname);
		fclose(fp);
		return 1;
	    } else {
	    	//printf("ppp_link not opened: %s\n", wan_ifname);
	        /* Open socket to kernel */
		if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
		{
		    return 0;
		}    
		/* Check for valid IP address */
		strncpy(ifr.ifr_name, wan_ifname, IFNAMSIZ);
		if (ioctl(s, SIOCGIFADDR, &ifr) >= 0) 
		{
    		    /* Otherwise we are probably configured */
		    close(s);
		    return 1;
	        }
	    }
	}
	unlink("/tmp/USRpr");
	
	if ((str = file2str("/var/run/USRpppoecd.pid"))) 
	{
	  	pid = atoi(str);
		free(str);
    		cprintf("Send SIGTERM (%d) to pppoecd's process %d...\n",SIGTERM, pid);
		kill(pid, SIGTERM);
    		usleep(500); /* wait for pppoe quit */
	}
	//cprintf("launch pppoecd vlan1 -D 1 -u -p -r 1492 -t 1492 -i 0 -U 0\n");
	system("pppoecd vlan1 -D 1 -u -p -r 1492 -t 1492 -i 0 -U 0");
	t = 0;
	
	while (t < 10)
	{
	    if ((fp = fopen("/tmp/USRpr", "r")))
		break;
	    t++;
	    //cprintf("pppoe_sleep: %d\n", t);
	    usleep(500); /* wait for pppoe server detection result */
	}
	if (fp)
	{
    	    if (fread(tmp, 1, 1, fp) == 1) 
	    {
		if (!strncmp(tmp, "1", 1))
		    ret = 1;
	    }
	    fclose(fp);
    	    unlink("/tmp/USRpr"); 
	}
	cprintf("pppoe result = %d\n",ret);

/*	if (nvram_match("wan_proto", "pppoe")) run_pppoe(); */

        return ret;
}

/*
static int
local_detect_dhcp(int eid, webs_t wp, int argc, char_t **argv)
{
    int unit,ret = 0;
    char tmp[100];
    char *str;
    int pid,t;
    FILE *fp;

    cprintf("Auto detecting DHCP server ...\n");

    if (!(fp = fopen("/tmp/USRdc", "w")))
	return 0;
    fwrite("check", 1, 5, fp); 
    fclose(fp);


    if ((unit = atoi(nvram_safe_get("wan_unit"))) < 0)
	unit = 0;
	
    snprintf(tmp, sizeof(tmp), "/var/run/udhcpc%d.pid", unit);
    if ((str = file2str(tmp))) 
    {
	pid = atoi(str);
	free(str);
        cprintf("Send SIGUSR2 (%d) <release> to udhcpc's process %d...\n",SIGUSR2, pid);
	kill(pid, SIGUSR2);
        sleep(1); // give udhcpcd the chance to catch the signal 
        cprintf("Send SIGUSR1 (%d) <renew> to udhcpc's process %d...\n",SIGUSR1, pid);
	kill(pid, SIGUSR1);
    } else {
	// udhcpd is not loaded need to run 
	system("udhcpc -d -i vlan1 -p /var/run/udhcpc0.pid -s /tmp/udhcpc");
    }

    t = 0;
	
    while (t < 5)
    {
        if ((fp = fopen("/tmp/USRdr", "r")))
	break;
        t++;
	sleep(1); // wait for pppoe server detection result 
    }
    unlink("/tmp/USRdc"); 
    
    cprintf("Read DHCP check result...\n");
    if (fp)
    {
	if (fread(tmp, 1, 1, fp) == 1) 
	{
	    if (!strncmp(tmp, "1", 1))
		ret = 1;
	}
	fclose(fp);
    	unlink("/tmp/USRdr"); 
    }
    cprintf("DHCP result = %d\n",ret);
    return ret;
}
*/
static int
local_detect_pppoe(int eid, webs_t wp, int argc, char_t **argv)
{
	if (local_check_pppoe(eid, wp, argc, argv) > 0) {
    		return 1;
	}
	return 0;
}

static int
local_default_detection(int eid, webs_t wp, int argc, char_t **argv)
{
	int ret = 0;
	char tmp1[30];
	char tmp2[30];	
    // Has not been configured yet, so do auto detection
    //if (local_detect_dhcp(eid, wp, argc, argv) > 0)
        // DHCP detected return 1 to the WEB page 
	sprintf(tmp1, nvram_safe_get("wan_proto"));
	sprintf(tmp2, nvram_safe_get("wan0_ipaddr"));
	//cprintf("wan_proto: %s\n", tmp1);
	//cprintf("wan_ipadrr: %s\n", tmp2);
	if (!strcmp(tmp1, "dhcp") && strcmp(tmp2, "0.0.0.0"))	    
		ret += websWrite(wp, "1");
	else if (!strcmp(tmp1, "static") && strcmp(tmp2, "0.0.0.0"))
		ret += websWrite(wp, "3");
    	else if (local_detect_pppoe(eid, wp, argc, argv) > 0)
        	ret += websWrite(wp, "2");
    	else  
        	ret += websWrite(wp, "0");
	
    	return ret;
}

/* Check for DHCP or PPPoE connections */
static int
local_auto_detection(int eid, webs_t wp, int argc, char_t **argv, int force)
{
    int ret = 0;
        
    if ((force) || (nvram_match("is_default", "1")) )
    {
	ret += local_default_detection(eid, wp, argc, argv);
    } else {
	if (nvram_match("wan_proto", "dhcp"))
	    ret += websWrite(wp, "21");
	    
    	else if (nvram_match("wan_proto", "pppoe")) 
	    ret += websWrite(wp, "22");
	
	else if (nvram_match("wan_proto", "static")) 
	    ret += websWrite(wp, "23");
    }
    return ret;
}

/*
void
run_pppoe(void)
{
    char sigterm[] = "-XX";

   // Write out leases file 
    sprintf(sigterm, "-%d", SIGTERM);
    eval("killall", sigterm, "udhcpc");
    local_run_pppoe();
}
*/

/* Check PPPoE login */
int
check_pppoelogin(webs_t wp, char *usr, char *pwd)
{
    return local_check_pppoelogin( wp, usr, pwd);
}


/* Check for DHCP or PPPoE connections */
int
wan_f_detection(int eid, webs_t wp, int argc, char_t **argv)
{
    force_detection = 1;
    
    return local_auto_detection(eid, wp, argc, argv, 1);
}

/* Check for DHCP or PPPoE connections */
int
wan_detection(int eid, webs_t wp, int argc, char_t **argv)
{
    force_detection = 0;

    return local_auto_detection(eid, wp, argc, argv, 0);
}


#endif	 /* __CONFIG_NAT__ */

#endif /* defined(linux) */
